@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2017 Jeffrey Stedfast
+ *  Copyright (C) 2000-2019 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -31,6 +31,8 @@
 
 #ifdef LIBIDN
 #include <idn2.h>
+#undef G_GNUC_UNUSED
+#undef G_GNUC_DEPRECATED
 #endif
 
 #include "internet-address.h"
